/* A set of functions to be used in testing the SIS from a web page.

   The web page should contain blank divs with the following id values:
    1. deviceListOutput
    2. debugLog
    3. functionButtons
    4. variableButtons

    External Calls:
        initWebPage() if there is anything to do
        logAdd(string) will add the string to the top of a time
            stamped list in a div with the id debugLog
        listAllDevices() will display a list of registered devices
            in a div with the id deviceListOutput and populate buttons
            and text boxes in functionButtons and variableButtons
        activeDeviceSet() set the window.activeDevice variable to be
             the selected device from spark.device[ ].


    (c) 2014 Jim Schrempp
*/
var g_activeDevice;
var g_attributes;   // how does this work when there is more than one core?

function initWebPage() {
   var a=1;
}


function logAdd(message) {
    // adds message to top of the div debugLog with a timestamp
    var currentTime = new Date();
    currentTime = currentTime.getHours() + ":" + currentTime.getMinutes() +
    ":" + currentTime.getSeconds() + "." + currentTime.getMilliseconds();
    currentTime = currentTime.valueOf();

    var logElement = document.getElementById("debugLog");

    var currentLog = logElement.innerHTML;
    currentLog = currentTime + " " + message + "<br>" + currentLog;

    logElement.innerHTML = currentLog;
}

function loginToSpark() {

    sparkLogin(function(data) {

        logAdd("back from sparklogin");

        listAllDevices();

    });
}

function listAllDevices() {
    //list devices
    logAdd("In listAllDevices");

    var devicesPr = spark.listDevices();
    devicesPr.then(
        function(devices){
            //we are here when the device information is available
            console.log('Devices: ', devices);

            //display all the devices on the web page
            var devlist = spark.devices;
            var output ="";
            devlist.forEach( function(entry) {
                var connectedValue = "";
                if (entry.connected) {
                    connectedValue = "OnLine"
                } else {
                    connectedValue = "OffLine"
                }
                output = output + "<b>name:</b> " + entry.name +
                    " <b>id:</b> " + entry.id + " <b>" + connectedValue + "</b><br>";
                });

            document.getElementById("deviceListOutput").innerHTML = output;

            // Now get all attributes
            var promiseX = spark.getAttributesForAll();
            promiseX.then (
                function(data) {
                    //we are here when the attribute information is available
                    logAdd("get attributes succeeded");
                    console.log("attributes: ", data);
                    g_attributes = data;

                    activeDeviceSet(); // set the device we will be using
                },
                function(err) {
                    logAdd("get attributes failed");
                }
        );

        },
        function(err) {
            logAdd('List devices call failed: ', err);
        }
    );
}

function activeDeviceSet(){
    // set the global device variable for other functions to use
    // eventually should pop a dialog for user to pick a device

    logAdd("In activeDeviceSet");
    var devList = spark.devices;
    logAdd("Active device is: " + devList[0].name);

    g_activeDevice = spark.devices[0]; // hard coded to first device

    console.log("Active device: " + g_activeDevice);

    console.log('Device name: ' + g_activeDevice.name);
    console.log('- connected?: ' + g_activeDevice.connected);

    listAttributes(); // now go find out what the core exports for us to use

    logAdd("---");

};

function listAttributes () {
    //retrive the functions and variables that the core supports
    //and populate buttons for each on the web page

    //console.log("global activedevice: ", g_activeDevice);
    //console.log("global attributes: ", g_attributes);

    g_attributes.forEach(function(entry) {

        console.log(entry);

        // create the buttons to call each available Spark.function
        var functionButtons = "";
        for (x in entry.functions){
            var functionName = entry.functions[x];
            logAdd("Function: " + functionName);
            functionButtons += '<button onclick="callFunction(' + "'" + functionName +
                "'" + ')"> Call ' + functionName + '</button>' + "&nbsp;&nbsp;" +
                'Input: <input type="text" id="' + functionName + 'Input' +
                '" size="15" >' +
                '<br>RtnCode: <input type="text" id="' + functionName + 'Return' +
                '" size="5" ><p>';
            //console.log(functionButtons);
        }
        document.getElementById("functionButtons").innerHTML = functionButtons;

        // create the buttons to retrieve each available Spark.variable
        var variableButtons = '';
        for (x in entry.variables){
            var variableName = entry.variables[x];
            logAdd("Variable: " + x + " type: " + variableName);
            variableButtons += '<button onclick="variableGet(' + "'" + x +
                "'" + ')"> Retrieve ' + x + '</button>' + "&nbsp;&nbsp;" +
                '<input type="text" id="' + x + 'Return' + '" size="15" ><p>';
        }
        document.getElementById("variableButtons").innerHTML = variableButtons;

    });

}

//-------  To call a Spark.function or retrieve a Spark.variable

function variableGet(variableToGet){
    //should make sure g_activeDevice is not null
    logAdd("Entered variableToGet for " + variableToGet);
    g_activeDevice.getVariable(variableToGet, function(err,data) {
        if(err){
            logAdd("error getting variable" + data); // is this right?
        } else {
            message = variableToGet + "current value: " + data.result;
            logAdd (message);
            document.getElementById(variableToGet + "Return").value = data.result;
        }
    });

}

function callFunction(functionToCall) {
    //should make sure g_activeDevice is not null
    logAdd("Entered variableSet for " + functionToCall);
    var stringToSend = document.getElementById(functionToCall + "Input").value;

    g_activeDevice.call(functionToCall, stringToSend, function(err,data) {
        if (err) {
            logAdd("Error calling variableSet" + data) // is this right
        } else {
            logAdd("Back from calling variableSet");
            document.getElementById(functionToCall + "Return").value = data.return_value;
            console.log("result ", data); // seems like return value is always 0...
        }
    });


}
